@@ -160,9 +160,9 @@ class Puppet::Pops::Parser::Lexer2
   # a letter a-z and may not contain dashes (\w includes letters, digits and _).
   #
   PATTERN_CLASSREF       = %r{((::){0,1}[A-Z][\w]*)+}
-  PATTERN_NAME           = %r{((::)?[a-z][\w]*)(::[a-z][\w]*)*}
+  PATTERN_NAME           = %r{^((::)?[a-z][\w]*)(::[a-z][\w]*)*$}
 
-  PATTERN_BARE_WORD      = %r{[a-z_](?:[\w-]*[\w])?}
+  PATTERN_BARE_WORD     = %r{((?:::){0,1}(?:[a-z_](?:[\w-]*[\w])?))+}
 
   PATTERN_DOLLAR_VAR     = %r{\$(::)?(\w+::)*\w+}
   PATTERN_NUMBER         = %r{\b(?:0[xX][0-9A-Fa-f]+|0?\d+(?:\.\d+)?(?:[eE]-?\d+)?)\b}
@@ -213,7 +213,7 @@ class Puppet::Pops::Parser::Lexer2
     @scanner = StringScanner.new(string)
     @locator = locator || Puppet::Pops::Parser::Locator.locator(string, '')
     @lexing_context[:escapes] = escapes || UQ_ESCAPES
-    @lexing_context[:uq_slurp_pattern] = (interpolate || !escapes.empty?) ? SLURP_UQ_PATTERN : SLURP_ALL_PATTERN
+    @lexing_context[:uq_slurp_pattern] = interpolate ? (escapes.include?('$') ? SLURP_UQ_PATTERN : SLURP_UQNE_PATTERN) : SLURP_ALL_PATTERN
   end
 
   # Convenience method, and for compatibility with older lexer. Use the lex_file instead.
@@ -274,7 +274,7 @@ class Puppet::Pops::Parser::Lexer2
     ctx   = @lexing_context
     queue = @token_queue
 
-    lex_error_without_pos("Internal Error: No string or file given to lexer to process.") unless scn
+    lex_error_without_pos(Puppet::Pops::Issues::NO_INPUT_TO_LEXER) unless scn
 
     scn.skip(PATTERN_WS)
 
@@ -533,17 +533,20 @@ class Puppet::Pops::Parser::Lexer2
           else
             # move to faulty position ('::<uc-letter>' was ok)
             scn.pos = scn.pos + 3
-            lex_error("Illegal fully qualified class reference")
+            lex_error(Puppet::Pops::Issues::ILLEGAL_FULLY_QUALIFIED_CLASS_REFERENCE)
           end
         else
-          # NAME or error
-          value = scn.scan(PATTERN_NAME)
+          value = scn.scan(PATTERN_BARE_WORD)
           if value
-            emit_completed([:NAME, value.freeze, scn.pos-before], before)
+            if value =~ PATTERN_NAME
+              emit_completed([:NAME, value.freeze, scn.pos-before], before)
+            else
+              emit_completed([:WORD, value.freeze, scn.pos - before], before)
+            end
           else
             # move to faulty position ('::' was ok)
             scn.pos = scn.pos + 2
-            lex_error("Illegal fully qualified name")
+            lex_error(Puppet::Pops::Issues::ILLEGAL_FULLY_QUALIFIED_NAME)
           end
         end
       else
@@ -575,30 +578,25 @@ class Puppet::Pops::Parser::Lexer2
       else
         # move to faulty position ([0-9] was ok)
         scn.pos = scn.pos + 1
-        lex_error("Illegal number")
+        lex_error(Puppet::Pops::Issues::ILLEGAL_NUMBER)
       end
 
     when 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
     'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '_'
-      value = scn.scan(PATTERN_NAME)
-      # NAME or false start because followed by hyphen(s), underscore or word
-      if value && !scn.match?(/^-+\w/)
+
+      value = scn.scan(PATTERN_BARE_WORD)
+      if value && value =~ PATTERN_NAME
         emit_completed(KEYWORDS[value] || [:NAME, value.freeze, scn.pos - before], before)
+      elsif value
+        emit_completed([:WORD, value.freeze, scn.pos - before], before)
       else
-        # Restart and check entire pattern (for ease of detecting non allowed trailing hyphen)
-        scn.pos = before
-        value = scn.scan(PATTERN_BARE_WORD)
-        # If the WORD continues with :: it must be a correct fully qualified name
-        if value && !(fully_qualified = scn.match?(/::/))
-          emit_completed([:WORD, value.freeze, scn.pos - before], before)
+        # move to faulty position ([a-z_] was ok)
+        scn.pos = scn.pos + 1
+        fully_qualified = scn.match?(/::/)
+        if fully_qualified
+          lex_error(Puppet::Pops::Issues::ILLEGAL_FULLY_QUALIFIED_NAME)
         else
-          # move to faulty position ([a-z_] was ok)
-          scn.pos = scn.pos + 1
-          if fully_qualified
-            lex_error("Illegal fully qualified name")
-          else
-            lex_error("Illegal name or bare word")
-          end
+          lex_error(Puppet::Pops::Issues::ILLEGAL_NAME_OR_BARE_WORD)
         end
       end
 
@@ -610,7 +608,7 @@ class Puppet::Pops::Parser::Lexer2
       else
         # move to faulty position ([A-Z] was ok)
         scn.pos = scn.pos + 1
-        lex_error("Illegal class reference")
+        lex_error(Puppet::Pops::Issues::ILLEGAL_CLASS_REFERENCE)
       end
 
     when "\n"
@@ -685,7 +683,7 @@ class Puppet::Pops::Parser::Lexer2
       true
 
     # Operands (that can be followed by DIV (even if illegal in grammar)
-    when :NAME, :CLASSREF, :NUMBER, :STRING, :BOOLEAN, :DQPRE, :DQMID, :DQPOST, :HEREDOC, :REGEX
+    when :NAME, :CLASSREF, :NUMBER, :STRING, :BOOLEAN, :DQPRE, :DQMID, :DQPOST, :HEREDOC, :REGEX, :VARIABLE, :WORD
       false
 
     else
