@@ -7,8 +7,10 @@ module Puppet
       information into the mount table. The actual behavior depends
       on the value of the 'ensure' parameter.
 
-      Note that if a `mount` receives an event from another resource,
-      it will try to remount the filesystems if `ensure` is set to `mounted`.
+      **Refresh:** `mount` resources can respond to refresh events (via
+      `notify`, `subscribe`, or the `~>` arrow). If a `mount` receives an event
+      from another resource **and** its `ensure` attribute is set to `mounted`,
+      Puppet will try to unmount then remount that filesystem.
 
       **Autorequires:** If Puppet is managing any parents of a mount resource ---
       that is, other mount points higher up in the filesystem --- the child
@@ -166,8 +168,11 @@ module Puppet
     end
 
     newproperty(:options) do
-      desc "Mount options for the mounts, as they would
-        appear in the fstab."
+      desc "Mount options for the mounts, comma-separated as they would appear
+      in the fstab on Linux. AIX options other than dev, nodename, or vfs may
+      be defined here. If specified, AIX options of account, boot, check, free,
+      mount, size, type, vol, log, and quota must be alphabetically sorted at
+      the end of the list."
 
       validate do |value|
         raise Puppet::Error, "option must not contain whitespace: #{value}" if value =~ /\s/
@@ -204,7 +209,7 @@ module Puppet
 
     newproperty(:dump) do
       desc "Whether to dump the mount.  Not all platform support this.
-        Valid values are `1` or `0`. or `2` on FreeBSD, Default is `0`."
+        Valid values are `1` or `0` (or `2` on FreeBSD). Default is `0`."
 
       if Facter.value(:operatingsystem) == "FreeBSD"
         newvalue(%r{(0|1|2)})
@@ -212,8 +217,6 @@ module Puppet
         newvalue(%r{(0|1)})
       end
 
-      newvalue(%r{(0|1)})
-
       defaultto {
         0 if @resource.managed?
       }
@@ -253,8 +256,16 @@ module Puppet
       newvalues(:true, :false)
       defaultto do
         case Facter.value(:operatingsystem)
-        when "FreeBSD", "Darwin", "AIX", "DragonFly", "OpenBSD"
+        when "FreeBSD", "Darwin", "DragonFly", "OpenBSD"
           false
+        when "AIX"
+          if Facter.value(:kernelmajversion) == "5300"
+            false
+          elsif resource[:device] and resource[:device].match(%r{^[^/]+:/})
+            false
+          else
+            true
+          end
         else
           true
         end
